C These are routines used by zefsa3d.for, scan3d.for, display3d.for,
C and topol.for.

C Written by Michael W. Deem August 1988.
C Modified by Michael W. Deem 30 March 1990.
C Modified by Michael W. Deem 9 July 1990.

C This routine returns the cross product of vectors a and b in vector c.
      subroutine CROSS(a, b, c)

      implicit none

      real*4 a(3), b(3), c(3)

      c(1) = a(2)*b(3) - a(3)*b(2)
      c(2) = a(3)*b(1) - a(1)*b(3)
      c(3) = a(1)*b(2) - a(2)*b(1)
      return
      end


C This routine moves the characters before the first delimiting space
C   in st1 to st2.
      subroutine SCANS(st1, st2, *)

      implicit none

      character*80 st1, st2

      integer*2 i, j

      i = len(st1)
      do 10 i = 1, len(st1)
        if (st1(i:i) .ne. ' ') goto 20
10      continue
      goto 50
20    do 30 j = i, len(st1)
        if (st1(j:j) .eq. ' ') goto 40
30      continue
      if (j-i .gt. len(st2)) goto 50
40    st2 = st1(i:j-1)
      if (j .le. len(st1)) then
        st1 = st1(j:len(st1))
      else
        st1 = ' '
      end if
      return
50    return 1
      end


C This function returns a non-blank line from UNIT unit.  Any error generates
C a return to the passed linenumber, presumably an error-handling
C routine in the calling procedure.
      subroutine GETLNE(unit, line, *)

      implicit none

      character*80 line
      integer*2 unit

      integer*2 i, j, offset

10      read(unit, 200, END=100, ERR=100) line 
C find starting brace
        i = index(line, '{')
20      if (i .eq. 0) goto 50 
C remove comments; stuff enclosed in braces; they cannot be nested
          offset = index(line, '}') - i
C check for closing brace
          if (offset .le. 0) goto 100
          do 30 j = i + offset + 1, len(line)
            line(i:i) = line(j:j)
30          i = i + 1 
          do 40 i = len(line)-offset, len(line)
40          line(i:i) = ' ' 
          i = index(line, '{')
        goto 20
C left adjust line
50      do 80 i = 1, 80 
          if (line(i:i) .ne. ' ') then
            do 60 j = 1, 80-i+1
60            line(j:j) = line(i+j-1:i+j-1) 
            do 70 j = 80-i,80
70            line(j:j) = ' ' 
            goto 90
          end if
80        continue 
90      if (line(1:1) .eq. ' ') goto 10 
      return
100   return 1 
200   format(A) 
      end


C This routine reads in the data from unit 2.
C If action = 0, it writes out unit 2 to unit 3.
C If action = 1, it writes out spline file data to unit 3.
C If action = 3, it reads position data.
C It uses unit 1 as a temporary file.
      logical function RDFILE(action)

      include 'zefsa3d.inc'

C action=0 for zefsa, 1 for scan3d, 2 for display3d, and 3 for topol
      integer*2 action

      logical RD_SPC, WRFILE
      integer*2 i, j, m, k, i2, i3, unit1, unit2, unit3
      character message*70, line*80, line2*80

c-------------------------------new definitions-------------------------c
        character*256 specfile
        character*256 buffer
        logical there
        integer nblength
c-----------------------------------------------------------------------c

      external GETLNE, SCANS, RD_SPC, WRFILE

      print *,'Reading input data file from unit 2'
      RDFILE = .false.
      unit3 = 3
      unit2 = 2
      unit1 = 1
      open(unit1,status='SCRATCH',ERR=260)
      if (action .eq. 0) then
C copy input file to top of position file
10        read(unit2,100,ERR=220,END=20) line
          write(unit3,*,ERR=230) line
          goto 10
20      message = 'Error rewinding input file'
        rewind(unit2,ERR=200)
      end if
C actually read in the input file
      if (action .ne. 3) then
        message =
     |  'Invalid specification of treatment of 4-coordinate structures'
        call GETLNE(unit2, line, 200)
        write_4 = .false.
        if (line(1:1) .eq. 'y' .or. line(1:1) .eq. 'Y') write_4 = .true.
        message =
     |'Invalid specification of number of saved 4-coordinate structures'
        read(unit2,*,ERR=200) max_wr4
        message =
     |'Invalid specification of energy of saved 4-coordinate structures'
        read(unit2,*,ERR=200) max_en4
        message =
     |   'Invalid specification of random number seed'
        read(unit2,*,ERR=200) r_seed
        message =
     |   'Invalid specification of splitting temperatures'
        read(unit2,*,ERR=200) temp_1, temp_2
        message =
     |   'Invalid specification of temperature decrement factors'
        read(unit2,*,ERR=200) tfact1, tfact2, tfact3
      end if
      message = 'Invalid degenerate/bonding distance'
      read(unit2,*,ERR=200) close, neighbor
      if (action .ne. 3) then
        message = 'Invalid specification of projection vectors'
        read(unit2,*,ERR=200) proj_n, ((proj_vec(i,j),i=1,3), j=1,3)
        message = 'Invalid number of pore shapes'
        read(unit2,*,ERR=200) n_pores
        n_cyl = 0
        n_sph = 0
        do 30 i = 1, n_pores
          write(message,120) 'Invalid pore shape ',i
          read(unit2,100,ERR=200) line
          call SCANS(line, line2, 240)
          write(unit1,100,ERR=250) line
          rewind(unit1,ERR=250)
          message = 'Invalid type of pore shape'
          if (line2 .ne. 'cylinder' .and. line2 .ne. 'sphere') goto 200
          if (line2 .eq. 'cylinder') then
            n_cyl = n_cyl + 1
            message = 'Too many pore cylinders'
            if (n_cyl .gt. MAX_PORE) goto 200
            write(message, 120) 'Invalid parameters for cylinder ',n_cyl
            read(unit1,*,ERR=200) 
     |     cyl_max(n_cyl), cyl_rad(n_cyl), (cyl_axis(j,n_cyl),j = 1,3)
          else
            n_sph = n_sph + 1
            message = 'Too many pore cylinders'
            if (n_sph .gt. MAX_PORE) goto 200
            write(message, 120) 'Invalid parameters for sphere ',n_sph
            read(unit1,*,ERR=200) sph_max(n_sph), sph_rad(n_sph)
          end if
30        continue
        message = 'Invalid jiggle step parameters'
        read(unit2,*,ERR=200) step_a, step_b, step_m
        message = 
     |     'Invalid weights for energy components'
        read(unit2,*,ERR=200) 
     |    wt_bond, wt_degn, wt_angl, wt_aang, wt_proj, wt_coor, wt_pore,
     |    wt_pxd
        read(unit2,*,ERR=200)
     |    p_bond, p_degn, p_angl, p_aang, p_proj, p_coor, p_pore, p_pxd
        message = 'Invalid specification of e_close array'
        read(unit2,*,ERR=200) (e_close(i), i = 0, MAX_CLOSE)
      end if
      message = 
     |'Invalid specification of number of atoms or symmetry operations'
      read(unit2,*,ERR=200) n_atoms, n_symm, n_actual
      message = 'Invalid number of unique atoms'
      if (n_atoms .lt. 1 .or. n_atoms .gt. MAX_UNIQ) goto 200
      message = 'Invalid number of symmetry operators'
      if (n_symm .lt. 1 .or. n_symm .gt. MAX_SYMM) goto 200
      message = 'Invalid number of actual atoms'
      if (n_actual .lt. 1 .or. n_actual .gt. n_atoms*n_symm) goto 200
      message = 'Too many total atoms'
      if (n_atoms*n_symm .gt. MAX_ATOMS) goto 200
      message = 'Invalid specification of bond coordination'
      read(unit2,*,ERR=200) (n_bond(i), i = 1, n_atoms)
C propagate coordination number to all atoms created by symmetry
C find maximum and minimum coordination number
      j = MAX_CLOSE
      k = 0
      do 40 i = n_symm*n_atoms, 1, -1
        n_bond(i) = n_bond((i-1)/n_symm+1)
        j = min(n_bond(i), j)
40      k = max(n_bond(i), k)
      message = 'Invalid bond coordination numbers'
      if (j .le. 0) goto 200
      if (k .ge. MAX_CLOSE) goto 200
      message = 'Invalid specification of cell constants'
      read(unit2,*,ERR=200) c_a, c_b, c_c
      message = 'Invalid specification of cell angles'
      read(unit2,*,ERR=200) c_alpha, c_beta, c_gamma
      c_alpha = c_alpha * PI / 180.0
      c_beta = c_beta * PI / 180.0
      c_gamma = c_gamma * PI / 180.0
      do 50 m = 1, n_symm
        write(message,'(A,I3)')
     |        'Invalid specification of symmetry operation ',m
        read(unit2,*,ERR=200)
     |      ((symm_m(j,i,m),j=1,3),i=1,3), (symm_v(i,m),i=1,3)
C        write(*,110) m,
C     |      ((symm_m(j,i,m),j=1,3),i=1,3), (symm_v(i,m),i=1,3)
50      continue
      k = MAX_TYPE
      j = 0
      if (action .ne. 3) then
C read in the types of the unique atoms
        do 60 i = 1, n_atoms
          write(message,'(A,I3)')
     |        'Invalid specification of atom type ',i
          read(unit2,*,ERR=200) a_type(i)
          k = min(k, a_type(i))
60        j = max(j, a_type(i))
        message = 'Invalid range of atom types'
        if (k .le. 0 .or. j .gt. MAX_TYPE .or. k .gt. j) goto 200

C write out the file for scan3d
        if (action .eq. 1) then
          if (.not. WRFILE()) goto 210
        end if


C read in the degenerate splines
        do 70 i = 1, j
          write(message,120)
     |        'Invalid specification of degenerate spline file name ',i
          call GETLNE(unit2, line, 200)
          if (action .eq. 1) write(3,100) line

c-----------------------------------------------------------------------c
      buffer="."
      specfile=buffer(1:nblength(buffer))//'/'
      specfile=specfile(1:nblength(specfile))//line(1:nblength(line))
      inquire(file=specfile,exist=there)
      if(.not.there)then
               print *,'Cannot find the degen_spectrum file'
               print *,'ZeFsA cannot proceed'
               goto210
      endif
c-----------------------------------------------------------------------c
70        if (.not. RD_SPC(specfile, 
     |                degn_n, degn_x0, degn_h, degn_y(1,i),
     |                degn_m(0,i), DEGN_KNOTS, 'degenerate')) goto 210

C read in the bond splines
        do 80 i = 1, j
          write(message,120)
     |        'Invalid specification of bond spline file name ',i,1
          call GETLNE(unit2, line, 200)
          if (action .eq. 1) write(3,100) line
          do 80 i2 = 1, j
            write(message,120)
     |       'Invalid specification of bond spline file name ',i,i2
            call SCANS(line, line2, 200)
c-----------------------------------------------------------------------c
      specfile=buffer(1:nblength(buffer))//'/'
      specfile=specfile(1:nblength(specfile))//line2(1:nblength(line2))
      inquire(file=specfile,exist=there)
      if(.not.there)then
               print *,'Cannot find the bond_spectrum file'
               print *,'ZeFsA cannot proceed'
               goto210
      endif
c-----------------------------------------------------------------------c
80          if (.not. RD_SPC(specfile, bond_n, bond_x0,
     |                  bond_h, bond_y(1,i,i2), bond_m(0,i,i2), 
     |                  BOND_KNOTS, 'bond')) goto 210

C read in the angle splines
        do 90 i = 1, j
          do 90 i2 = 1, j
            write(message,120)
     |        'Invalid specification of angle spline file name ',i,i2,1
            call GETLNE(unit2, line, 200)
            if (action .eq. 1) write(3,100) line
            do 90 i3 = 1, j
              write(message,120)
     |       'Invalid specification of angle spline file name ',i,i2,i3
              call SCANS(line, line2, 200)
c-----------------------------------------------------------------------c
      specfile=buffer(1:nblength(buffer))//'/'
      specfile=specfile(1:nblength(specfile))//line2(1:nblength(line2))
      inquire(file=specfile,exist=there)
      if(.not.there)then
               print *,'Cannot find the bond_spectrum file'
               print *,'ZeFsA cannot proceed'
               goto210
      endif
c-----------------------------------------------------------------------c
90            if (.not. RD_SPC(specfile, angl_n, angl_x0,
     |                    angl_h, angl_y(1,i,i2,i3), angl_m(0,i,i2,i3),
     |                    ANGL_KNOTS, 'angle')) goto 210

C read in the average angle splines
        do 94 i = 1, j
          write(message,120)
     |     'Invalid specification of average angle spline file name ',i
          call GETLNE(unit2, line, 200)
          if (action .eq. 1) write(3,100) line
c-----------------------------------------------------------------------c
      specfile=buffer(1:nblength(buffer))//'/'
      specfile=specfile(1:nblength(specfile))//line(1:nblength(line))
      inquire(file=specfile,exist=there)
      if(.not.there)then
               print *,'Cannot find the bond_spectrum file'
               print *,'ZeFsA cannot proceed'
               goto210
      endif
c-----------------------------------------------------------------------c
94        if (.not. RD_SPC(specfile, aang_n, aang_x0, 
     |            aang_h, aang_y(1,i), aang_m(0,i), 
     |            ANGL_KNOTS,'average angle')) goto 210
      end if

      if (action .eq. 3) then
        do 96 m = 1, n_atoms
          write(message,120)
     |        'Invalid specification of atom position ',m
          read(unit2,*,ERR=200)
     |      (a_pos(i,m),i=1,3)
96        continue
      end if
      if (action .eq. 0 .or. action .eq. 3) close(unit2)
      close(unit1)
      RDFILE = .true.
      return
100   format(A)
110   format(1X, I3, 100 F6.2)
120   format(A, 3I3)
200   print *,'Error: ',message
210   close(unit2)
      if (action .eq. 0) close (unit3)
      close(unit1)
      return
220   print *,'Error reading 80 character text lines from input file'
      goto 210
230   print *,
     | 'Error writing 80 character text lines to position output file'
      goto 210
240   print *,'Error: Invalid pore shape ', i
      goto 210
250   print *,'Error writing to temporary file'
      goto 210
260   print *,'Error opening temporary file'
      goto 210
      end


C This function writes out the input data to unit 3
      logical function WRFILE()

      include 'zefsa3d.inc'

      integer*2 i, j, m, unit
      character line*80

      print *,'Writing input data to unit 3'
      WRFILE = .false.
      unit = 3
      line = 'n'
      if (write_4) line = 'y'
      write(unit,100,ERR=200) line
      write(unit,*,ERR=200) max_wr4
      write(unit,*,ERR=200) max_en4
      write(unit,*,ERR=200) r_seed
      write(unit,*,ERR=200) temp_1, temp_2
      write(unit,*,ERR=200) tfact1, tfact2, tfact3
      write(unit,*,ERR=200) close, neighbor
      write(unit,*,ERR=200) proj_n, ((proj_vec(i,j),i=1,3), j=1,3)
      write(unit,*,ERR=200) n_pores
      do 10 i = 1, n_cyl
10      write(unit,*) 'cylinder ',cyl_max(i), cyl_rad(i),
     |                         (cyl_axis(j,i),j=1,3)
      do 20 i = 1, n_sph
20      write(unit,*) 'sphere ',sph_max(i), sph_rad(i)
      write(unit,*,ERR=200) step_a, step_b, step_m
      write(unit,*,ERR=200)
     |      wt_bond, wt_degn, wt_angl, wt_aang, wt_proj, wt_coor,
     |      wt_pore, wt_pxd
      write(unit,*,ERR=200)
     |      p_bond, wt_degn, p_angl, p_aang, p_proj, p_coor,
     |      p_pore, p_pxd
      write(unit,*,ERR=200) (e_close(i), i = 0, MAX_CLOSE)
C The damn Cray dies at runtime with the next line
      write(unit,*,ERR=200) n_atoms, n_symm, n_actual
      write(unit,*) (n_bond((i-1)*n_symm+1),i=1,n_atoms)
      write(unit,*,ERR=200) c_a, c_b, c_c
      write(unit,*,ERR=200)
     |       c_alpha*180.0/PI, c_beta*180.0/PI, c_gamma*180.0/PI
      do 30 m = 1, n_symm
      write(unit,110,ERR=200)
     |      ((symm_m(j,i,m),j=1,3),i=1,3), (symm_v(i,m),i=1,3)
30      continue
      do 60 i = 1, n_atoms
60      write(unit,*,ERR=200) a_type(i)
      WRFILE = .true.
      return
100   format(A)
110   format(1X, 100 F8.4)
200   print *,'Error writing output file'
      close(unit)
      return
      end


C This routine reads in the spline data from file.
      logical function RD_SPC(file, n, x0, h, y, m, KNOTS, type)

      implicit none

      character*(*) file, type
      integer*2 n, KNOTS
      real*4 x0, h, y(KNOTS), m(0:KNOTS)

      character*70 message
      integer*2 i, unit

      RD_SPC = .false.
      unit = 8
      write(message,100) type
C read in the energy spectra (spline parameters)
      open(unit,file=file,status='old',ERR=200)
      write(message,110) type
      read(unit,*,ERR=200) n
      read(unit,*,ERR=200) x0
      read(unit,*,ERR=200) h
      if (n+1 .gt. KNOTS) then
      message =
     |   'Too many knots in bond energy spectrum'
        close(unit)
        goto 210
      end if
      do 10 i = 1, n+1
10      read(unit,*,ERR=200) y(i), m(i-1)
      close(unit)
      RD_SPC = .true.
      return
100   format('Error opening ',A,' spectrum')
110   format('Error reading ',A,' spectrum')
200   print *,'Error: ',message
210   return
220   print *,'Error reading 80 character text lines from input file'
      goto 210
230   print *,
     | 'Error writing 80 character text lines to position output file'
      goto 210
      end


C This routine writes out the energy spectra as a stream of numbers
      subroutine WR_SPC(file, n, x0, h, y, m, KNOTS)

      implicit none

      character*(*) file
      integer*2 n, KNOTS
      real*4 x0, h, y(KNOTS), m(0:KNOTS)

      integer*2 i
      real*4 tmp2
      real*4 x1, x2, x_len

C write out bond energy curve
      open(8,file=file,status='unknown')
      do 10 x_len = x0, x0 + n*h,
     |              n*h / 1000.0
        i = (x_len - x0) / h + 1.0
        i = max(min(i, n), 1)
        x1 = x_len - (x0 + (i-1)*h)
        x2 = h - x1
        tmp2 = (m(i-1) * x2*x2*x2 +
     |               m(i) * x1*x1*x1  +
     |               (6.0*y(i) - m(i-1)*h*h)*x2 +
     |               (6.0*y(i+1) - m(i)*h*h)*x1)
     |               / (6.0 * h)
10    write(8,*) x_len, tmp2
      close(8)
      return
      end


C This routine prints out the data read in.
      subroutine DUMP1()

      include 'zefsa3d.inc'

      integer*2 i,j,k

      print *,'n_atoms ',n_atoms
      print *,'n_symm ',n_symm
      print *,'n_actual ',n_actual
      print *,'c_a, c_b, c_c ',c_a, c_b, c_c
      print *,'c_alpha, c_beta, c_gamma', c_alpha, c_beta, c_gamma
      do 20 i = 1, n_symm
        do 10 j = 1, 3
10        print *, (symm_m(k,j,i), k = 1, 3)
        print *, (symm_v(j,i), j = 1, 3)
20      print *
      return
      end


C This function prints out the atomic position & grid information.
      subroutine DUMP2()

      include 'zefsa3d.inc'

      integer*2 i,j, k, l
      real*4 x,y,z
      external TO_XYZ

      print *,'i, TO_XYZ(ta_pos)'
      do 10 i = 1, n_atoms*n_symm
        call TO_XYZ(x,y,z,ta_pos(1,i), ta_pos(2,i), ta_pos(3,i))
10      write(*,100) i, x, y, z
      do 20 k = 1, nz_grid
        do 20 j = 1, ny_grid
          do 20 i = 1, nx_grid
            print *,'grid ',i,j,k
            print *,'l, ta2_pos, a_from, ta_from, ta_iden'
              do 20 l = grid(1,i,j,k), grid(1,i,j,k)+grid(2,i,j,k)-1
                write(*,100) l, ta2_pos(1,l), ta2_pos(2,l),
     |                       ta2_pos(3,l), a_from(l),
     |                       ta_from(l), ta_iden(l)
20              continue
      return
100   format(I4, 3F12.5, 3I4)
      end


C This function prints out the atoms created by symmetry.
      subroutine DUMP3()

      include 'zefsa3d.inc'

      integer*2 i

      print *,'i, ta_pos'
      do 10 i = 1, n_atoms*n_symm
10      write(*,100) i, ta_pos(1,i), ta_pos(2,i), ta_pos(3,i)
      return
100   format(1X, I4, 3F12.5)
      end


C This routine prints out the grid data
      subroutine DUMP4()

      include 'zefsa3d.inc'

      integer*2 i, j, k, n

      n = grid(1, nx_grid, ny_grid, nz_grid) +
     |    grid(2, nx_grid, ny_grid, nz_grid) - 1
      do 10 i = 1, n
        print *, i, a_from(i), ta_from(i), ta_iden(i), cell_fr(i)
10      print *, (ta2_pos(j,i), j = 1,3)
      do 20 k = 1, nz_grid
        do 20 j = 1, ny_grid
          do 20 i = 1, nx_grid
20          print *, i, j, k, grid(1,i,j,k), grid(2,i,j,k)
      return
      end


C This routine puts all atoms in the 0,0,0 unit cell
      subroutine MOD_POS()

      include 'zefsa3d.inc'

      integer*2 i

      do 10 i = 1, n_atoms
        a_pos(1,i) = mod(mod(a_pos(1,i)+1.0, 1.0)+1.0, 1.0)
        a_pos(2,i) = mod(mod(a_pos(2,i)+1.0, 1.0)+1.0, 1.0)
10      a_pos(3,i) = mod(mod(a_pos(3,i)+1.0, 1.0)+1.0, 1.0)
      return
      end


C This routine determines useful constants.
      subroutine CONST()

      include 'zefsa3d.inc'

      integer*2 i, j, k, l
      real*4 a1, b1, b2, c1, c2, c3

C compute constants in transformation of (a,b,c) to (x,y,z)
      a1 = 1.0
      b1 = cos(c_gamma)
      b2 = sin(c_gamma)
      c1 = cos(c_beta)
      c2 = (cos(c_alpha) - cos(c_gamma)*cos(c_beta)) / sin(c_gamma)
      c3 = sqrt(1.0 - (c1*c1 + c2*c2))
C determine matrix to convert from a,b,c to x,y,z
C a = <c_x1, 0,    0>
C b = <c_x2, c_y1, 0>
C c = <c_x3, c_y2, c_z1>
      c_x1 = c_a * a1
      c_x2 = c_b * b1
      c_x3 = c_c * c1
      c_y1 = c_b * b2
      c_y2 = c_c * c2
      c_z1 = c_c * c3
C determine cell volume
      c_vol = c_x1 * c_y1 * c_z1
C determine matrix to convert x,y,z to a,b,c
C x = <c_a1, 0,    0>
C y = <c_a2, c_b1, 0>
C z = <c_a3, c_b2, c_c1>
      c_c1 = 1.0 / c_z1
      c_b1 = 1.0 / c_y1
      c_b2 = -c_y2 / (c_y1*c_z1)
      c_a1 = 1.0 / (c_x1)
      c_a2 = -c_x2 / (c_x1*c_y1)
      c_a3 = c_x2*c_y2 / (c_x1*c_y1*c_z1) - c_x3/(c_x1*c_z1)
C determine reciprocal lattice vectors
C unit cell vector i . reciprocal lattice vector j = delta ij
C h = <rec_x1, rec_y2, rec_z3>
C k = <     0, rec_y1, rec_z2>
C l = <     0,      0, rec_z1>
      rec_z1 = 1.0 / c_z1
      rec_y1 = 1.0 / c_y1
      rec_z2 = -c_y2*rec_y1 / c_z1
      rec_x1 = 1.0 / c_x1
      rec_y2 = -c_x2*rec_x1 / c_y1
      rec_z3 = - (c_x3*rec_x1 + c_y2*rec_y2) / c_z1
C compute distances used in bond calcuations
      bond = bond_x0 + bond_h*bond_n
      close2 = close*close
      l = 0
      do 10 i = -1, 1
        do 10 j = -1, 1
          do 10 k = -1, 1
            l = l + 1
            transl(1,l) = k
            transl(2,l) = j
10          transl(3,l) = i
      return
      end


C This routine determines the multiplication table of the symmetry group.
C It also checks to see that the input symmetry operations represent a group.
      logical function GROUP()
 
      include 'zefsa3d.inc'
 
      logical bad
      integer*2 i, j, k, k2, l, i0
      real*4 m(3,3), v(3), sum, eps, delta
      parameter (eps = 1.0E-5)
C delta allows for a sum just shy of 1.0
C not that 3*delta MUST be less than eps
      parameter (delta = 1.0E-6)
 
C A group is defined by four properties: 1) closure; 2) associativity; 3) an
C identity element; and 4) an inverse for each element
      print *,
     |   'Computing group multiplication table and testing closure'
C make all translations to next positively displaced cell only
      do 10 i = 1, n_symm
        symm_v(1,i) = mod(mod(symm_v(1,i),1.0)+1.0+delta,1.0)
        symm_v(2,i) = mod(mod(symm_v(2,i),1.0)+1.0+delta,1.0)
10      symm_v(3,i) = mod(mod(symm_v(3,i),1.0)+1.0+delta,1.0)
C compute multiplication table
C compute k, where Oi Oj x = Ok x
      do 80 i = 1, n_symm
        do 80 j = 1, n_symm
C Oi(Oj(x)) = Oi(Mjx + Vj) = Mi(Mjx + Vj) + Vi = MiMjx + (MiVj + Vj)
C remember matrices are defined col,row
          do 20 k = 1,3
            do 20 l = 1,3
20            m(l,k) = symm_m(1,k,i)*symm_m(l,1,j) +
     |                 symm_m(2,k,i)*symm_m(l,2,j) +
     |                 symm_m(3,k,i)*symm_m(l,3,j)
          do 30 k = 1, 3
30          v(k) = symm_m(1,k,i)*symm_v(1,j) +
     |             symm_m(2,k,i)*symm_v(2,j) +
     |             symm_m(3,k,i)*symm_v(3,j) + symm_v(k,i)
C make all translations to next positively displaced cell only
          do 40 k = 1, 3
40          v(k) = mod(mod(v(k),1.0)+1.0+delta, 1.0)
C          write(*,210) i,j,
C     |        ((m(k2,k),k2=1,3),k=1,3), (v(k),k=1,3)
C now find which operation this is
C that is, compare matrices and vectors
          k = 0
50          k = k+1
            sum = 0.0
            do 70 k2 = 1, 3
              do 60 l = 1, 3
60              sum = sum + abs(m(l,k2) - symm_m(l,k2,k))
70            sum = sum + abs(v(k2) - symm_v(k2,k))
            if (sum .gt. eps .and. k .lt. n_symm) goto 50
          if (sum .gt. eps) print *,i,j,' not closed'
          bad = bad .or. sum .gt. eps
80        mult_t(i,j) = k
C      do 90 i = 1, n_symm
C90      write(*,200) (mult_t(i,j), j = 1, n_symm)
C finding identity element
      print *,'Searching for identity element'
      do 100 i = 1, n_symm
        sum = abs(symm_m(1,1,i)-1.0) + abs(symm_m(2,1,i)) +
     |                abs(symm_m(3,1,i)) + abs(symm_v(1,i)) +
     |        abs(symm_m(1,2,i)) + abs(symm_m(2,2,i)-1.0) +
     |                abs(symm_m(3,2,i)) + abs(symm_v(2,i)) +
     |        abs(symm_m(1,3,i)) + abs(symm_m(2,3,i)) +
     |                abs(symm_m(3,3,i)-1.0) + abs(symm_v(3,i))
        if (sum .le. eps) goto 110
100     continue
      print *,'No identity element found'
      bad = .true.
      goto 190
110   i0 = i
C check for a unique inverse to each element
      print *,'Checking for inverses'
      do 130 i = 1, n_symm
        do 120 j = 1, n_symm
          if (mult_t(j,i) .eq. i0 .and. mult_t(i,j) .eq. i0) goto 130
120       continue
C no inverse found for element i
        bad = .true.
        print *,'No inverse found for ',i
        goto 190
130     continue
      print *,'Checking associativity'
      do 140 i = 1, n_symm
        do 140 j = 1, n_symm
          do 140 k = 1, n_symm
             if (mult_t(mult_t(i,j),k) .ne. mult_t(i,mult_t(j,k))) then
               bad = .true.
               print *,i,j,k,' not associative'
               goto 190
             end if
140          continue
190   if (bad) print *,
     |    'The symmetry operations do not represent a group'
      GROUP = .not. bad
      return
200   format(1X, 100 I3)
C210   format(1X, 2 I3, 12F6.2)
      end


C this subroutine determines the constants TO_XYZ, TO_XY, TO_YZ, & TO_XZ need.
      subroutine I_XYZ()

      include 'zefsa3d.inc'

      x_off = min(0.0, c_x2)
      x_off = min(x_off, x_off + c_x3)
      x_max = max(c_x1, c_x1 + c_x2)
      x_max = max(x_max, x_max + c_x3) - x_off
      y_off = min(0.0, c_y2)
      y_max = max(c_y1, c_y1 + c_y2) - y_off
      z_max = c_z1
      xyz_max = max(x_max+2.0*bond, y_max+2.0*bond, z_max+2.0*bond)
      return
      end


C This function initializes the grid used in the grid search.
C The general idea is to only consider atoms in the parallelpiped that
C   (in real space) encloses the unit cell.  This parallelpiped is
C   incremented by bond on each side to allow for translated atoms.
C   That is, it is an isotropically scaled version of the unit cell.
C The parallelpiped is cut into smaller parallelpipeds to help locate
C   atoms close to any given atom.
      logical function I_GRID()

      include 'zefsa3d.inc'

      real*4 a(3), b(3), c(3), x

      external CROSS

      I_GRID = .false.
      print *,'Initializing grid'
C 1,1,1 is corner of the grid at -bond,-bond,-bond
C grid squares will be at least bond * bond * bond in size
C get c_c cross c_b
      a(1) = c_x3
      a(2) = c_y2
      a(3) = c_z1
      b(1) = c_x2
      b(2) = c_y1
      b(3) = 0.0
      call CROSS(a, b, c)
      x = sqrt(c(1)*c(1) + c(2)*c(2) + c(3)*c(3))
      a0_grid = bond*(c_a1*c(1) + c_a2*c(2) + c_a3*c(3)) / x
C get c_a cross c_c
      a(1) = c_x1
      a(2) = 0.0
      a(3) = 0.0
      b(1) = c_x3
      b(2) = c_y2
      b(3) = c_z1
      call CROSS(a, b, c)
      x = sqrt(c(1)*c(1) + c(2)*c(2) + c(3)*c(3))
      b0_grid = bond*(c_b1*c(2) + c_b2*c(3)) / x
C get c_b cross c_a
      a(1) = c_x2
      a(2) = c_y1
      a(3) = 0.0
      b(1) = c_x1
      b(2) = 0.0
      b(3) = 0.0
      call CROSS(a, b, c)
      x = sqrt(c(1)*c(1) + c(2)*c(2) + c(3)*c(3))
      c0_grid = bond * c_c1*c(3) / x
      a1_grid = 1.0 - a0_grid
      b1_grid = 1.0 - b0_grid
      c1_grid = 1.0 - c0_grid
      x_grid = x_max+bond
      y_grid = y_max+bond
      z_grid = z_max+bond
      nx_grid = x_max/bond + 2.0
      ny_grid = y_max/bond + 2.0
      nz_grid = z_max/bond + 2.0
      dx_grid = (x_max + 2.0*bond) / nx_grid
      dy_grid = (y_max + 2.0*bond) / ny_grid
      dz_grid = (z_max + 2.0*bond) / nz_grid
      if (ny_grid .gt. MAX_GRID .or. nx_grid .gt. MAX_GRID .or.
     |    nz_grid .gt. MAX_GRID) goto 10
      I_GRID = .true.
      return
10    print *,
     |  'Cell constants are too large for grid used in grid search'
      return
      end


C This subroutine returns the actual coordinates, given cell
C   coordinates.
      subroutine TO_XYZ(x, y, z, c_x, c_y, c_z)

      include 'zefsa3d.inc'

      real*4 x, y, z, c_x, c_y, c_z

      x = c_x1*c_x + c_x2*c_y + c_x3*c_z - x_off
      y = c_y1*c_y + c_y2*c_z - y_off
      z = c_z1*c_z
      return
      end


C This subroutine returns the scaled, paper coordinates, given cell
C   coordinates, for xy projection.
C z is set to 0.0.
      subroutine TO_XY(x, y, c_x, c_y, c_z)

      include 'zefsa3d.inc'

      external TO_XYZ

      real*4 x, y, z, c_x, c_y, c_z

      call TO_XYZ(x, y, z, c_x, c_y, 0.0)
      x = (x+pap_x1) * PAP_SIZ / pap_max
      y = (y+pap_y) * PAP_SIZ / pap_max
      return
      end


C This subroutine returns the scaled, paper coordinates, given cell
C   coordinates, for yz projection.
C z is set to 0.0.
      subroutine TO_YZ(y, z, c_x, c_y, c_z)

      include 'zefsa3d.inc'

      external TO_XYZ

      real*4 x, y, z, c_x, c_y, c_z

      call TO_XYZ(x, y, z, 0.0, c_y, c_z)
      y = (y+pap_x) * PAP_SIZ / pap_max
      z = (z+NEIGHBOR) * PAP_SIZ / pap_max
      return
      end


C This subroutine returns the scaled, paper coordinates, given cell
C   coordinates, for xz projection.
C z is set to 0.0.
      subroutine TO_XZ(x, z, c_x, c_y, c_z)

      include 'zefsa3d.inc'

      external TO_XYZ

      real*4 x, y, z, c_x, c_y, c_z

      call TO_XYZ(x, y, z, c_x, 0.0, c_z)
      x = (x+pap_x2) * PAP_SIZ / pap_max
      z = (z+NEIGHBOR) * PAP_SIZ / pap_max
      return
      end


C This function returns true if the passed point is on the grid.  
C i,j is the location of the grid pointer to the given point.
      logical function TO_GRD(i,j,k,x,y,z)

      include 'zefsa3d.inc'

      integer*2 i,j, k
      real*4 x, y, z

      real*4 a,b,c

      a = c_a1*x + c_a2*y + c_a3*z
      b = c_b1*y + c_b2*z
      c = c_c1*z
      TO_GRD = a .ge. a0_grid .and. a .le. a1_grid .and.
     |         b .ge. b0_grid .and. b .le. b1_grid .and.
     |         c .ge. c0_grid .and. c .le. c1_grid
      i = min(max(int((x+bond)/dx_grid)+1,1),nx_grid)
      j = min(max(int((y+bond)/dy_grid)+1,1),ny_grid)
      k = min(max(int((z+bond)/dz_grid)+1,1),nz_grid)
      return
      end


C This routine transforms the projection directions
C to the internal orthogonal angstrom settings

      subroutine PR_CON()

      include 'zefsa3d.inc'

      integer*2 i
      real*4 x, y, z

       do 100, i=1, proj_n
C TO_XYZ MACRO
          x = c_x1*proj_vec(1,i) + c_x2*proj_vec(2,i) 
     |           + c_x3*proj_vec(3,i)  - x_off
          y = c_y1*proj_vec(2,i) + c_y2*proj_vec(3,i)  - y_off
          z = c_z1*proj_vec(3,i) 
          proj_vec(1,i) = x 
          proj_vec(2,i) = y 
100       proj_vec(3,i) = z 
      return
      end


C This routine transforms the cylinder axis directions
C to the internal orthogonal angstrom settings.
C The position is not shifted by <-x_off, -y_off, 0>, though.
C It normalizes the axis to unit length.
      subroutine CYL_CON()

      include 'zefsa3d.inc'

      integer*2 i
      real*4 x, y, z, len

       do 10, i = 1, n_cyl
C TO_XYZ MACRO
          x = c_x1*cyl_axis(1,i) + c_x2*cyl_axis(2,i) 
     |           + c_x3*cyl_axis(3,i)
          y = c_y1*cyl_axis(2,i) + c_y2*cyl_axis(3,i)
          z = c_z1*cyl_axis(3,i) 
          len = sqrt(x*x + y*y + z*z)
          cyl_axis(1,i) = x / len
          cyl_axis(2,i) = y / len
10        cyl_axis(3,i) = z / len
      return
      end


C This routine creates the table of grid pointers.
      logical function MK_GRD()

      include 'zefsa3d.inc'

      logical dummy(27), ltmp, ltmp2
      integer*2 m, i, j, i0, j0, k0, sum, tmp, m2, k, l
      integer*2 from(MAX_TA+1), from2(MAX_TA+1), iden(MAX_TA+1)
      integer*2 n_square(MAX_GRID, MAX_GRID, MAX_GRID),
     |          jtmp(MAX_TA), itmp(MAX_TA)
      integer*2 tmp_pnt(3,MAX_TA+1), from3(MAX_TA+1)
      real*4 a, b, c, tmp_pos(3,MAX_TA+1), xtmp(27), ytmp(27),
     |       ztmp(27)

      MK_GRD = .false.
      m = n_atoms*n_symm
C initialize pointers
      do 10 k = 1, nz_grid
        do 10 j = 1, ny_grid
          do 10 i = 1, nx_grid
10      n_square(i,j,k) = 0
C Create translated atoms in neighboring cells.
C Determine number of atoms under each pointer.
C Also create list of all points on grid.
C Elements with same grid j, same grid k and increasing grid i
C   will be contiguous in grid array.
      m2 = 0
      do 30 i = 1, m
C find atoms that should go into grid
        k = 0
        do 20 j = 1, 27
C TO_XYZ MACRO
          a = ta_pos(1,i) + transl(1,j)
          b = ta_pos(2,i) + transl(2,j)
          c = ta_pos(3,i) + transl(3,j)
          xtmp(j) = c_x1*a + c_x2*b + c_x3*c - x_off
          ytmp(j) = c_y1*b + c_y2*c - y_off
          ztmp(j) = c_z1*c
          dummy(j) = a .ge. a0_grid .and. a .le. a1_grid .and.
     |                b .ge. b0_grid .and. b .le. b1_grid .and.
     |                c .ge. c0_grid .and. c .le. c1_grid
          l = 0
          if (dummy(j)) l = 1
20        k = k + l
        if (m2 + k .gt. MAX_TA) goto 90
C compact array into big array
          do 30 j = 1, 27
            if (dummy(j)) then
              m2 = m2 + 1
              itmp(m2) = i
              jtmp(m2) = j
              tmp_pos(1,m2) = xtmp(j)
              tmp_pos(2,m2) = ytmp(j)
              tmp_pos(3,m2) = ztmp(j)
            end if
30          continue
C now add atoms to storage
      do 40 i = 1, m2
        i0 = min(max(int((tmp_pos(1,i)+bond)/dx_grid)+1,1),nx_grid)
        j0 = min(max(int((tmp_pos(2,i)+bond)/dy_grid)+1,1),ny_grid)
        k0 = min(max(int((tmp_pos(3,i)+bond)/dz_grid)+1,1),nz_grid)
        tmp_pnt(1,i) = i0
        tmp_pnt(2,i) = j0
        tmp_pnt(3,i) = k0
        ltmp = mod(itmp(i)-1, n_symm) .eq. 0
        ltmp2 = jtmp(i) .eq. 14
C check to see if unique T-atom (created by identity symmetry operation)
        from(i) = 0
        if (ltmp .and. ltmp2) from(i) = (itmp(i)-1)/n_symm + 1
C see if this is the atom in the unit cell
        from2(i) = 0
        if (ltmp2) from2(i) = itmp(i)
        from3(i) = jtmp(i)
        iden(i) = itmp(i)
40      continue
C update number of pointers in each grid cube
      do 50 i = 1, m2
        i0 = tmp_pnt(1,i)
        j0 = tmp_pnt(2,i)
        k0 = tmp_pnt(3,i)
50      n_square(i0, j0, k0) = n_square(i0, j0, k0) + 1
C create the pointers: where pointed to, number there
C do distribution counting
      sum = 1
      do 60 k = 1, nz_grid
        do 60 j = 1, ny_grid
          do 60 i = 1, nx_grid
            tmp = n_square(i,j,k)
            grid(2,i,j,k) = tmp
            grid(1,i,j,k) = sum
            n_square(i,j,k) = sum
60          sum = sum + tmp
C distribution count sort list so that elements with a common pointer
C are contiguous in the array
C create mapping from temporary storage to final storage
      do 70 i = 1, m2
        i0 = tmp_pnt(1,i)
        j0 = tmp_pnt(2,i)
        k0 = tmp_pnt(3,i)
        m = n_square(i0,j0,k0)
        itmp(i) = m
70      n_square(i0,j0,k0) = n_square(i0,j0,k0) + 1
C actually move the data to final storage
      do 80 i = 1, m2
        m = itmp(i)
        ta2_pos(1,m) = tmp_pos(1,i)
        ta2_pos(2,m) = tmp_pos(2,i)
        ta2_pos(3,m) = tmp_pos(3,i)
        a_from(m) = from(i)
        ta_from(m) = from2(i)
        cell_fr(m) = from3(i)
80      ta_iden(m) = iden(i)
      MK_GRD = .true.
      return
90    print *,'Too many atoms'
      return
      end


C This function returns the energy of the system.
      real*4 function ENERGY(pos, cyl, sph, minocc, maxocc, coord4, nt,
     |                       evec, move_pos, rel_t)

      include 'zefsa3d.inc'
      include 'zefsa3d-b.inc'

      logical move_pos, coord4
      integer*2 minocc, maxocc, nt
      real*4 pos(3,MAX_UNIQ), cyl(3, MAX_PORE), sph(3,MAX_PORE),
     |       evec(8), rel_t

      logical j_degen, ignore, MK_GRD, i_near(MAX_TA),
     |        MK_DGN
      integer*2 i, j, k, l, n, m, i0, j0, k0, l0, l1, n_close,
     |          n_degen, n_merged, 
     |          n_total, list(MAX_TA), itype,
     |          neartype(MAX_CLOSE), nearfrom(MAX_CLOSE), jtype, ktype
      real*4 sum, x, y, z, x2, y2, z2, near(3,MAX_CLOSE),
     |       a, b, c, tmp2, x_len, tmp3, tmp4, x1,
     |       ANGLE, CVMGT, LEN_2, ang_tmp, proj_conv, aang, ENERGY2

      external DO_SYM, MK_GRD, MK_DGN, ENERGY2

      LEN_2(x,y,z,j) = (x - ta2_pos(1,j))**2 +
     |                 (y - ta2_pos(2,j))**2 +
     |                 (z - ta2_pos(3,j))**2


C gamma = arccos(A dot B / (|A| |B|))
      ANGLE(x,y,z,x2,y2,z2) = 180.0 / PI * acos((x*x2 + y*y2 + z*z2) / 
     |                       (sqrt(x*x + y*y + z*z) *
     |                        sqrt(x2*x2 + y2*y2 + z2*z2) + 1.0E-4))


      coord4 = .true.
      proj_conv=1.0
C number of atoms allowed to mark as degenerate (= number degenerate / 2)
      n_degen = (n_atoms*n_symm - n_actual)
      if (.not. MK_DGN(pos)) stop
C determine total number of atoms in the grid
      n_total = grid(1, nx_grid, ny_grid, nz_grid) +
     |          grid(2, nx_grid, ny_grid, nz_grid) - 1
C find number of merged atoms
      n_merged = 0
      do 10 i = 1, n_atoms*n_symm
10     if (degen(i)) n_merged = n_merged + 1
C clear neighbor list
      do 20 j = 1, n_total
20     i_near(j) = .false.
C loop over unique atoms to find energy*n_atoms/(n_actual) of configuration
      maxocc = 0
      minocc = MAX_CLOSE
      evec(1) = 0.0
      evec(2) = 0.0
      evec(3) = 0.0
      evec(4) = 0.0
      evec(5) = 0.0
      evec(6) = 0.0
      evec(7) = 0.0
      evec(8) = 0.0
      n_oxy = 0
      do 170 i = 1, n_atoms
        itype = a_type(i)
C loop over elements that are near this atom to find bond energy
        a = pos(1,i)
        b = pos(2,i)
        c = pos(3,i)
C TO_XYZ MACRO
        x = c_x1*a + c_x2*b + c_x3*c - x_off
        y = c_y1*b + c_y2*c - y_off
        z = c_z1*c
C TO_GRD MACRO
        i0 = min(max(int((x+bond)/dx_grid)+1,1),nx_grid)
        j0 = min(max(int((y+bond)/dy_grid)+1,1),ny_grid)
        k0 = min(max(int((z+bond)/dz_grid)+1,1),nz_grid)
        l0 = max(i0-1,1)
        l1 = min(i0+1, nx_grid)
        n = 0
C z loop
        do 100 j = max(k0-1,1), min(k0+1, nz_grid)
C y loop
          do 100 k = max(j0-1,1), min(j0+1, ny_grid)
C x loop; loop over all elements under this pointer
            do 100 l = grid(1,l0,k,j), grid(1,l1,k,j)+
     |                                grid(2,l1,k,j)-1
              n = n + 1
100           list(n) = l
        do 110 j = 1, n
          l = list(j)
C create vector from atom l--the conventional origin--to atom i
          x2 = x - ta2_pos(1,l)
          y2 = y - ta2_pos(2,l)
          z2 = z - ta2_pos(3,l)
          x_len = sqrt(x2*x2 + y2*y2 + z2*z2) + 1.0E-4
          m = ta_iden(l)
          jtype = a_type((m-1)/n_symm+1)
C to be degenerate to atom j, it must degenerate, symmetrically equivalent
C   to atom i, and close
          j_degen = degen(m) .and. ((m-1)/n_symm+1 .eq. i)
     |                   .and. (x_len .le. close)
C check to make sure this is not atom i and is not degenerate (unless it's
C   degenerate to atom j)
          ignore = (a_from(l) .eq. i) .or.
     |             degen(m) .and. .not. j_degen
C compute energies from cubic splines
C CVMGT(p1, p2, logical) = p1 if logical is true; else, p2,
C so CVMGT can be written on any machine.
          m = (x_len - bond_x0) / bond_h + 1.0
          m = max(min(m, bond_n), 1)
          x1 = min(x_len - (bond_x0 + (m-1)*bond_h), bond_h)
          x2 = bond_h - x1
          tmp2 = (bond_m(m-1,itype,jtype) * x2*x2*x2 +
     |           bond_m(m,itype,jtype) * x1*x1*x1  +
     |           (6.0*bond_y(m,itype,jtype) -
     |               bond_m(m-1,itype,jtype)*bond_h*bond_h)*x2 +
     |           (6.0*bond_y(m+1,itype,jtype) -
     |               bond_m(m,itype,jtype)*bond_h*bond_h)*x1)
     |           / (6.0 * bond_h)
          m = (x_len - degn_x0) / degn_h + 1.0
          m = max(min(m, degn_n), 1)
          x1 = min(x_len - (degn_x0 + (m-1)*degn_h), degn_h)
          x2 = degn_h - x1
          tmp3 = d_fact(i) * (degn_m(m-1,itype) * x2*x2*x2 +
     |           degn_m(m,itype) * x1*x1*x1  +
     |           (6.0*degn_y(m,itype) -
     |                   degn_m(m-1,itype)*degn_h*degn_h)*x2 +
     |           (6.0*degn_y(m+1,itype) -
     |                   degn_m(m,itype)*degn_h*degn_h)*x1)
     |           / (6.0 * degn_h)
          tmp3 = CVMGT(0.0, tmp3, .not. j_degen .or. ignore)
          tmp2 = CVMGT(0.0, tmp2, j_degen .or. ignore)
          evec(1) = evec(1) + tmp2
          evec(2) = evec(2) + tmp3
          i_near(l) = (.not. (j_degen .or. ignore)) .and.
     |                    x_len .lt. neighbor
110       continue
        n_close = 0
C collect neighboring (unique) atoms found
        do 120 j = 1, n_total
          if (i_near(j)) then
            n_close = min(n_close+1, MAX_CLOSE)
            near(1,n_close) = ta2_pos(1,j)
            near(2,n_close) = ta2_pos(2,j)
            near(3,n_close) = ta2_pos(3,j)
            k = (ta_iden(j)-1)/n_symm+1
            nearfrom(n_close) = k
            neartype(n_close) = a_type(k)
            i_near(j) = .false.
          end if
120       continue
C create oxygens and their occupancies for PXD purposes
        if (pxd .and. .not. nooxy) then
          do 125 j = 1, n_close
            jtype = nearfrom(j)
              x2 = near(1,j) + x_off
              y2 = near(2,j) + y_off
              z2 = near(3,j)
              n_oxy = n_oxy + 1
              k = n_oxy + n_atoms
              if (k .gt. MAX_UNIQ) goto 300
              pos(1, k) = 0.5*(a + c_a1*x2 + c_a2*y2 + c_a3*z2)
              pos(2, k) = 0.5*(b + c_b1*y2 + c_b2*z2)
              pos(3, k) = 0.5*(c + c_c1*z2)
              a_occup(k) = 0.5 * a_occup(i)
125         continue
        end if
C compute angle strain
        aang = 0.0
        do 130 j = 1, n_close-1
          jtype = neartype(j)
          do 130 k = j+1, n_close
            ktype = neartype(k)
            x_len = ANGLE(near(1,k)-x, near(2,k)-y, near(3,k)-z,
     |                    near(1,j)-x, near(2,j)-y, near(3,j)-z)
            aang = aang + x_len
            m = (x_len - angl_x0) / angl_h + 1.0
            m = max(min(m, angl_n), 1)
            x1 = x_len - (angl_x0 + (m-1)*angl_h)
            x2 = angl_h - x1
            tmp2 = (angl_m(m-1,itype,jtype,ktype) * x2*x2*x2 +
     |             angl_m(m,itype,jtype,ktype) * x1*x1*x1  +
     |             (6.0*angl_y(m,itype,jtype,ktype) -
     |               angl_m(m-1,itype,jtype,ktype)*angl_h*angl_h)*x2 +
     |             (6.0*angl_y(m+1,itype,jtype,ktype) -
     |                 angl_m(m,itype,jtype,ktype)*angl_h*angl_h)*x1)
     |             / (6.0 * angl_h)
130         evec(3) = evec(3) + tmp2
C compute average angle term
        aang = aang / (n_close*(n_close-1)/2)
        m = (x_len - aang_x0) / aang_h + 1.0
        m = max(min(m, aang_n), 1)
        x1 = x_len - (aang_x0 + (m-1)*aang_h)
        x2 = aang_h - x1
        tmp2 = (aang_m(m-1,itype) * x2*x2*x2 +
     |         aang_m(m,itype) * x1*x1*x1  +
     |         (6.0*aang_y(m,itype) -
     |            aang_m(m-1,itype)*aang_h*aang_h)*x2 +
     |         (6.0*aang_y(m+1,itype) -
     |             aang_m(m,itype)*aang_h*aang_h)*x1)
     |         / (6.0 * aang_h)
        evec(4) = evec(4) + tmp2
C compute contribution for degree of 3-connectedness in projection(s)
C assume a harmonic, (0.0 -theta)**2 potential
        tmp2=0.0
        do 160 k = 1, proj_n
C          print *, 'k, proj_vec(x,k) ', k, (proj_vec(m,k),m=1,3) 
          x1=90.0
          do 140 j = 1, n_close
            print *, 'j, near(x,j) ', j, (near(m,j),m=1,3)
            ang_tmp = ANGLE(near(1,j)-x, near(2,j)-y, near(3,j)-z,
     |                    proj_vec(1,k), proj_vec(2,k), proj_vec(3,k) )
            ang_tmp = mod(ang_tmp+ 360.0, 180.0)
            ang_tmp = min(ang_tmp, 180.0 - ang_tmp)
C            print *, 'ang_tmp  ',ang_tmp
140         x1 = min(ang_tmp, x1)
          do 150 j = 1, n_close-1
            do 150 l = j+1, n_close
              ang_tmp = ANGLE(near(1,l)-near(1,j), near(2,l)-near(2,j),
     |                    near(3,l)-near(3,j),
     |                    proj_vec(1,k), proj_vec(2,k), proj_vec(3,k) )
              ang_tmp = mod(ang_tmp+ 360.0, 180.0)
              ang_tmp = min(ang_tmp, 180.0 - ang_tmp)
C              print *, 'j,l, ang_tmp  ',j,l,ang_tmp
150           x1 = min(ang_tmp, x1)
160       tmp2 = tmp2 + x1*x1
C        print *, 'projection contribution  ', tmp2
        tmp2 = proj_conv*tmp2
        evec(5) = evec(5) + tmp2
C compute neighbor contribution
        evec(6) = evec(6) + e_close(n_close-n_bond((i-1)*n_symm+1)+4)
        coord4 = coord4 .and. n_bond((i-1)*n_symm+1) .eq. n_close
        minocc = min(minocc, n_close)
        maxocc = max(maxocc, n_close)
170     continue
C compute pore shape contribution
      sum = 0.0
C loop over all nondegenerate atoms in unit cell
      do 220 i = 1, n_symm*n_atoms
        if (degen(i)) goto 220
        a = ta_pos(1,i)
        b = ta_pos(2,i)
        c = ta_pos(3,i)
C modified TO_XYZ MACRO
        x = c_x1*a + c_x2*b + c_x3*c
        y = c_y1*b + c_y2*c
        z = c_z1*c
        tmp2 = 0.0
C get sphere contribution
        do 190 j = 1, n_sph
          tmp3 = 1.0E30
C find closest sphere to atom i
          do 180 k = 1, 27
C modified TO_XYZ MACRO
            a = sph(1,j) + transl(1,k)
            b = sph(2,j) + transl(2,k)
            c = sph(3,j) + transl(3,k)
            x2 = x - (c_x1*a + c_x2*b + c_x3*c)
            y2 = y - (c_y1*b + c_y2*c)
            z2 = z - (c_z1*c)
180         tmp3 = min(tmp3, x2*x2 + y2*y2 + z2*z2)
190       sum = sum + sph_max(j)*
     |                  exp(-(tmp3/(sph_rad(j)*sph_rad(j)))**3)
C get cylinder contribution
        do 210 j = 1, n_cyl
          tmp3 = 1.0E30
C find closest cylinder to atom i
          do 200 k = 1, 27
C modified TO_XYZ MACRO
            a = cyl(1,j) + transl(1,k)
            b = cyl(2,j) + transl(2,k)
            c = cyl(3,j) + transl(3,k)
            x2 = x - (c_x1*a + c_x2*b + c_x3*c)
            y2 = y - (c_y1*b + c_y2*c)
            z2 = z - (c_z1*c)
C            write(21,*) x2, y2, z2, (cyl_axis(l,j),l=1,3)
C subtract off projection onto cylinder axis
            tmp4 = x2*cyl_axis(1,j)+y2*cyl_axis(2,j)+z2*cyl_axis(3,j)
            x2 = x2 - tmp4*cyl_axis(1,j)
            y2 = y2 - tmp4*cyl_axis(2,j)
            z2 = z2 - tmp4*cyl_axis(3,j)
C            write(21,*) x2, y2, z2
200         tmp3 = min(tmp3, x2*x2 + y2*y2 + z2*z2)
          tmp2 = cyl_max(j) *
     |                  exp(-(tmp3/(cyl_rad(j)*cyl_rad(j)))**3)
C          write(20,*) wt_pore, cyl_max(j), tmp3, cyl_rad(j)
C          print *, tmp3, tmp2
210       sum = sum + tmp2
220     continue
      sum = sum / n_symm
      evec(7) = sum
C add in PXD term
      n_atoms = n_atoms + n_oxy
      if (pxd) evec(8) = ENERGY2(pos, rel_t)
      n_atoms = n_atoms - n_oxy
      evec(1) = wt_bond*evec(1) * rel_t**p_bond
      evec(2) = wt_degn*evec(2) * rel_t**p_degn
      evec(3) = wt_angl*evec(3) * rel_t**p_angl
      evec(4) = wt_aang*evec(4) * rel_t**p_aang
      evec(5) = wt_proj*evec(5) * rel_t**p_proj
      evec(6) = wt_coor*evec(6) * rel_t**p_coor
      evec(7) = wt_pore*evec(7) * rel_t**p_pore
      evec(8) = wt_pxd*evec(8) * rel_t**p_pxd
      nt = n_atoms*n_symm - n_merged
      ENERGY = evec(1)+evec(2)+evec(3)+evec(4)+evec(5)+evec(6)+
     |         evec(7)+evec(8)
      return
300   print *,'Too many oxygens were created in ENERGY'
      stop
      end


C This routine marks degenerate atoms
      logical function MK_DGN(pos)

      include 'zefsa3d.inc'
      include 'zefsa3d-b.inc'

      real*4 pos(3,MAX_UNIQ)

      logical dummy, MK_GRD
      integer*2 i, j, k, l, m, i0, j0, k0, k1, l0, l1
      integer*2 n_degen, n_merged, to_merge, merge(MAX_SYMM-1)
      integer*2 n_total
      real*4 x, y, z
      real*4 a, b, c
      real*4 LEN_2
      external DO_SYM, MK_GRD

      LEN_2(x,y,z,j) = (x - ta2_pos(1,j))**2 +
     |                 (y - ta2_pos(2,j))**2 +
     |                 (z - ta2_pos(3,j))**2

C number of atoms allowed to mark as degenerate (= number degenerate / 2)
      n_degen = (n_atoms*n_symm - n_actual)
      n_merged = 0
      do 10 i = 1, n_atoms*n_symm
        degn_eq(i) = i
10      degen(i) = .false.
      do 20 i = 1, n_atoms
        d_fact(i) = 0.0
20      a_occup(i) = 1.0
      call DO_SYM(pos)
      dummy = MK_GRD()
      if (.not. dummy) goto 90
      n_total = grid(1, nx_grid, ny_grid, nz_grid) +
     |          grid(2, nx_grid, ny_grid, nz_grid) - 1
C don't try to merge if it can't be done
      if (n_degen .lt. n_symm/2) goto 90
C attempt to merge seemingly degenerate atoms
C loop through unique atoms in cell
C only equivalent atoms can be merged
C all symmetry equivalents will have the same number of degenerate neighbors
      do 80 i = 1, n_atoms
        to_merge = 0
C get close, equivalent atoms to atom j using grid search
        a = pos(1,i)
        b = pos(2,i)
        c = pos(3,i)
C TO_XYZ MACRO
        x = c_x1*a + c_x2*b + c_x3*c - x_off
        y = c_y1*b + c_y2*c - y_off
        z = c_z1*c
C TO_GRD MACRO
        i0 = min(max(int((x+bond)/dx_grid)+1,1),nx_grid)
        j0 = min(max(int((y+bond)/dy_grid)+1,1),ny_grid)
        k0 = min(max(int((z+bond)/dz_grid)+1,1),nz_grid)
        l0 = max(i0-1,1)
        l1 = min(i0+1,nx_grid)
C mark close, symmetrically equivalent atoms to atom i degenerate
C z loop
        do 30 k = max(k0-1,1), min(k0+1, nz_grid)
C y loop
          do 30 k1 = max(j0-1,1), min(j0+1, ny_grid)
C x loop; loop over all elements under these same-i pointers
            do 30 l = grid(1,l0,k1,k), grid(1,l1,k1,k)+
     |                                 grid(2,l1,k1,k)-1
              m = ta_iden(l)
C watch out for atom i
C atoms degenerate in this unit cell won't be degenerate in others
C that is all atoms found hereby are distinct (disregarding symmetry)
              if (((m-1)/n_symm+1 .eq. i) .and. (a_from(l) .ne. i)
     |            .and. (LEN_2(x,y,z,l) .le. close2)) then
C keep a record of what has been merged
                to_merge = to_merge + 1
                merge(to_merge) = m
              end if
30          continue
          j0 = 0
          if (to_merge .ne. 0) then
C mark all degenerate atoms
            l0 = (i-1)*n_symm
            do 50 j = l0+1, l0+n_symm
              if (.not. degen(j)) then
C mark this cluster (using symmetry operation j-l0)
C as the cluster around atom i is marked
                do 40 k = 1, to_merge
                  l = l0+mult_t(j-l0,merge(k)-l0)
                  degn_eq(l) = j
40                degen(l) = .true.
              end if
50            continue
C determine number of degenerate atoms just marked
          do 60 j = l0+2, l0+n_symm
60          if (degen(j)) j0 = j0 + 1
        end if
        n_merged = n_merged + j0
C see if this was an acceptable merge
        if (n_merged .gt. n_degen) then
C this atom is not degenerate, so reset all "merged" flags
          n_merged = n_merged - j0
          do 70 j = l0+2, l0+n_symm
            degn_eq(j) = j
70          degen(j) = .false.
        else if (to_merge .ne. 0) then
C compute number of degenerate atoms per nondegenerate atom /  number of
C   degenerate atoms that will be found
          d_fact(i) = real(j0) / (real(n_symm - j0)*real(to_merge))
          a_occup(i) = real(n_symm-j0) / real(n_symm)
        end if
80      continue
90    MK_DGN = dummy
      return
      end


C This routine determines if the right number of atoms are degenerate
      logical function CHK_DGN()

      include 'zefsa3d.inc'

      logical ok
      integer*2 i, j

      j = 0
      do 10 i = 1, n_atoms*n_symm
        if (degen(i)) j = j + 1
10      continue
C MWD 16 September 1999 fix n_acutal
      n_actual = n_atoms*n_symm - j
      ok = j .eq. n_atoms*n_symm - n_actual
      print *,'Atoms found: ',n_atoms*n_symm - j
      if (.not. ok) print *,'N actual atoms not found on first pass'
      CHK_DGN = ok
      return
      end


C This routine creates the atoms from the symmetry.
      subroutine DO_SYM(pos)

      include 'zefsa3d.inc'

      integer*2 i, j, m
      real*4 pos(3,MAX_UNIQ)

      m = 0
      do 10 i = 1, n_atoms
        do 10 j = 1, n_symm
          m = m + 1
          ta_pos(1,m) = symm_m(1,1,j) * pos(1,i) +
     |                  symm_m(2,1,j) * pos(2,i) +
     |                  symm_m(3,1,j) * pos(3,i) +
     |                  symm_v(1,j)
          ta_pos(2,m) = symm_m(1,2,j) * pos(1,i) +
     |                  symm_m(2,2,j) * pos(2,i) +
     |                  symm_m(3,2,j) * pos(3,i) +
     |                  symm_v(2,j)
          ta_pos(3,m) = symm_m(1,3,j) * pos(1,i) +
     |                  symm_m(2,3,j) * pos(2,i) +
     |                  symm_m(3,3,j) * pos(3,i) +
     |                  symm_v(3,j)
C put the atoms back in the unit cell, if necessary
          ta_pos(1,m) = mod(mod(ta_pos(1,m)+1.0, 1.0)+1.0, 1.0)
          ta_pos(2,m) = mod(mod(ta_pos(2,m)+1.0, 1.0)+1.0, 1.0)
10        ta_pos(3,m) = mod(mod(ta_pos(3,m)+1.0, 1.0)+1.0, 1.0)
      return
      end

c------------------------------Function to:----------------------------c
c------------------------------return the non blank lenght of a string-c
       integer function nblength(string)
       character*(*) string
       ilength=len(string)
       do 10 i=1,ilength
           if(string(i:i).eq.' ')then
                nblength=i-1
                return
           endif
   10  continue
       nblength=i
       return
       end
c------------------------------end of nblength-------------------------c


