/*

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html


  (C) 1997 Marco Falcioni 

  UCLA

  scattering parameter data

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "error.h"
#include "misc.h"
#include "vectors.h"

#include "SP_entry.h"

struct SP_entry * new_SP_entry(void) 
{
  unsigned i;
  struct SP_entry * This;

  This = (struct SP_entry *) Malloc(sizeof(struct SP_entry));

  This->label = 0;
  This->elem = 0;
  
  for(i = 0; i < 10; i++) {
    This->params[i] = 0.0;
  }
  This->atomic_n = 0;
  return This;
}

void delete_SP_entry(struct SP_entry * me) 
{
  if(me == 0) return;
  if(me->label != 0) Free(me->label);
  if(me->elem != 0) Free(me->elem);
  Free(me);
  return;
}


void print_SP_entry(struct SP_entry * me, FILE * stream) 
{
  if(stream == 0) {
    push_generic_error("print_SP_entry","(nil) arg passed");
    exit_clean();
  }
  
  if(me == 0) {
    fprintf(stream,"Nil SP_entry\n");
    return;
  }
  
  fprintf(stream,"%s %s ",me->label, me->elem);
  
  vector_dump_d(stream, me->params, 10, "%f", " ");
  
  fprintf(stream," %d\n", me->atomic_n);
  
  return;
}



struct SP_entry * read_SP_entry(struct SP_entry * me, FILE * stream)
{
  char buffer[1024];
  char word[5];
  char * w = 0;

  if(me == 0 || stream == 0) {
    push_generic_error("read_SP_entry","(nil) args passed");
    exit_clean();
  }

  fgets(buffer, sizeof(buffer), stream);

  if(feof(stream)) return 0;

  w = clean_line(buffer);
  
  while(isspace((int)*w)) w++;  /* skip blank chars */ 

  sscanf(w, "%s", word);
  me->label = (char *) Malloc((strlen(word) + 1) * sizeof(char));
  strcpy(me->label, word);

  while(!isspace((int) *w)) w++; /* skip word we just read */
  while(isspace((int) *w)) w++;  /* skip blank chars */

  strncpy(word,w,4);
  me->elem = (char *) Malloc((strlen(word) + 1) * sizeof(char));
  strcpy(me->elem, word);
  me->elem[4] = '\0';
  while(!isspace((int) *w)) w++; /* skip word we just read */
  while(isspace((int) *w)) w++;  /* skip blank chars */

  if(w[1] == '+' || w[1] == '-') { /* skip charge */
    while(!isspace((int) *w)) w++; /* skip word we just read */
    while(isspace((int) *w)) w++;  /* skip blank chars */
  }

  if(sscanf(w,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d",
	    &me->params[0], &me->params[1],
	    &me->params[2], &me->params[3], &me->params[4],
	    &me->params[5], &me->params[6], &me->params[7],
	    &me->params[8], &me->params[9], &me->atomic_n) != 11) {
    push_generic_error("read_SP_entry","error reading line");
    exit_clean();
  }

  w = get_line(buffer,sizeof(buffer),stream); /* what is this? */
  if(w == 0) {
    push_generic_error("read_SP_entry","this should not happen");
  }
  
  return me;
}

