/*

  Marco Falcioni (C) 1997

  Atoms; Let's see where this can lead us.
  
  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#ifndef __atoms_h
#define __atoms_h

/* includes */
#include <stdio.h>


#define LABELSIZE 5

/* data */
struct Cell;
struct Chain;

struct Atom {

  /* position dependent */
  double ra;
  double rb;
  double rc;
  double rx;
  double ry;
  double rz;

  double merge_d;
  unsigned merge_n;

  unsigned consistent;
  unsigned mod;

  /* position independent */

  double DW_fac;
  double occupancy;
  double Radius; /* this is the atomic radius according to Kittel page 78 */
  char label[LABELSIZE]; /* the name of the atom */

  /* bookkeeping */

  unsigned n_bonds;
  unsigned SP_tag;
  unsigned tag;
  unsigned root; /* the unique atom to which this atom refers to */
  unsigned refs; /* number of Link structs pointing to Atom */
  struct Atom * next;
};


/* function prototypes */
struct Atom * new_Atom(void);
void delete_Atom(struct Atom * A);
void write_Atom(const struct Atom * A, FILE * stream);
void print_Atom(const struct Atom * A, FILE * stream);
void shift_Atom(struct Atom * A, double * delta);
void clone_Atom(struct Atom * A, const struct Atom * B);
void move_Atom(struct Atom * A, double * pos);
void mod_Atom(struct Atom * A);
void xyz_Atom(struct Atom * A, const struct Cell * C);
void set_label_Atom(struct Atom * A, const char * string);

#endif
