#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include "error.h"
#include "resources.h"
#include "stringlib.h"
#include "constants.h"

#include "misc.h"
#include "functions.h"
#include "grid.h"
#include "chain.h"
#include "link.h"
#include "atoms.h"
#include "spline.h"
#include "params.h"
#include "rad_pattern.h"
#include "oogl.h"
#include "positions.h"
#include "verbose.h"
#include "conf.h"

int main (int argc, char ** argv)
{
  struct Params * P = 0;
  struct Cell * C = 0;
  struct Positions * T = 0;
  struct Positions * T2 = 0;
  struct Sym_group * G = 0;
  struct Grid * Gr = 0;
  struct Rad_pattern * RP = 0;
  double delta[3];
  struct Conf * Min;
  char line[1024];
  long steps;
  struct Link * a;
  int uatom;
  struct string_list * options;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);

  read_data(&P, &T, &C, &G, &RP, stdin);

  get_line(line, sizeof(line), stdin);
  sscanf(line, "%d %lf %lf %lf %ld", &uatom, 
	 &delta[0], &delta[1], &delta[2], &steps);

  Min = new_Conf(T->n_unique);

  Srand(P->seed);

  Gr = new_Grid(P, C, 0);
  T2 = new_Positions(T->n_unique, T->n_symmetry, T->n_actual);
 
  for(a = T->unique->head; uatom > 0; uatom --) a = a->next;

  while (steps > 0) {
    copy_Positions(T,T2);
    symmetrize_Positions(T2,G);
    mod_Positions(T2);   
    
    merge_positions(T2,P,C);
    if(T2->distinct->length < P->n_actual) continue;
    clean_Grid(Gr);
    fill_Grid(Gr,T2,C);
    if(energy (P, C, T2, Gr, Min) < max_double) fprint_Conf(Min, stdout);
    shift_Atom(a->atom, delta);
    steps --;
  }




  return 0;
  
}

