/* 


   Gerard Jungman (jungman@nnn.lanl.gov) (C) 1997
   Created: [GJ] Wed Jan 15 20:59:25 EST 1997
 

*/
#ifndef ERROR_H_
#define ERROR_H_

#include <stdio.h>

#define Error_None_         0
#define Error_General_      1
#define Error_Domain_       2
#define Error_ConvFail_    10
#define Error_Alloc_      100

#define _LIBNAME_ "libgjm-1.3"

/* Connect/disconnect error stream to a given physical stream. */
extern int connect_error_stream(FILE * error_stream, char * name);
extern int disconnect_error_stream(void);

/* Retrieve the last error code pushed on
to the error stream. */
extern int error_code(void);

/* Retrieve explanatory message for a given error code.
   Returns a pointer to a static string.
 */
extern const char * error_explain(int code);

/* Push a libgjm error onto the error stream.  */
extern void push_error(const char * mess, int code);

/* Push a generic message onto the error stream. */
extern void push_generic_error(const char * prefix, const char * mess);

/* Push a generic warning onto the error stream. */
extern void push_generic_warning(const char * prefix, const char * mess);

/* Flush the error stream. */
extern void flush_error_stream(void);

/* Send fatal signal down error stream. (suicide signal) */
extern void exit_clean(void);


#endif /* ERROR_H_ */
