/*

  Marco Falcioni (C) 1997

  header file for symmetry group data structures and functions


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#ifndef __group_h
#define __group_h

#include <stdio.h>
#include "atoms.h"
#include "misc.h"

/* 

   Structure Group_el defined 

   A generic group element has a matrix and a translation vector.
   we could bundle up other properties, if we need them. 
   the element of a group is made of 3x3 matrices and this is always implied.

*/

struct Group_el {
  double ** W;
  double * w;
};

/* function prototypes */
/* allocate a new group element */
struct Group_el * new_Group_el ( void );
/* free a group element */
void delete_Group_el (struct Group_el * a);

/* multiply two group elements and put the result in a previously 
   allocated element */
void mult_Group_el (struct Group_el * a,
			const struct Group_el * b,
			const struct Group_el * c);

/* compare two group elements and return a difference (sum of abs of 
   difference component by component */ 
double comp_Group_el (const struct Group_el * a,
			const struct Group_el * b);

/* apply group element to a point and put result in first passed 
   argument, assuming it was allocated already */

void apply_Group_el (const struct Group_el * g, 
		     const struct Atom * A, struct Atom * B);
/* same as above, but no translation */
void apply_Group_el_noT (const struct Group_el * g, 
		     const struct Atom * A, struct Atom * B);

struct Group_el * read_Group_el(FILE * stream);
struct Group_el * read_Group_el_XTL(FILE * stream);

void write_Group_el(const struct Group_el * a, FILE * stream);
void write_Group_el_XTL(const struct Group_el * a, FILE * stream);

/* non allocating copy a = b */
void copy_Group_el(struct Group_el * a, const struct Group_el * b);

#endif
