/*

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html


*/
/* 
   (C) Marco Falcioni UCLA 1998

   link methods: nothing here...


*/


#include <stdlib.h>
#include "error.h"
#include "atoms.h"
#include "misc.h"
#include "link.h"


long n_links = 0;

struct Link * new_Link(struct Atom * A)
{
  struct Link * me;
  if (A == 0) {
    push_generic_error("new_Link","linking a (nil) atom?");
    return 0; /* this is nice: no atom = no link */
   }

  me = (struct Link *) Malloc(sizeof(struct Link));

#ifdef DEBUG
  ++n_links;
  printf("Links: %ld\n",n_links);
#endif

  me->next = 0;
  me->prev = 0;
  me->atom = A;
  me->key = 0.0;
  A->refs++;
  return me;
}


void delete_Link(struct Link * me)
{
  if(me==0) return;
  if(me->atom != 0) {
    me->atom->refs--;
    if(me->atom->refs==0) delete_Atom(me->atom);
  } else {
    push_generic_error("delete_Link","freeing Link pointing to no Atom!?");
  }
  Free(me);

#ifdef DEBUG
  -- n_links;
  printf("Links: %ld\n",n_links);
#endif
  return;
}

void print_Link(const struct Link * me, FILE * stream)
{
  if(me == 0 || stream == 0) {
    push_generic_error("print_Link","(nil) args passed");
    return;
  }
  fprintf(stream,"Link at %p {",me);
  fprintf(stream,"Link->");
  print_Atom(me->atom, stream);
  fprintf(stream,"  link->next = %p",me->next);
  fprintf(stream," link->prev = %p",me->prev);
  fprintf(stream," link->key = %f}\n",me->key);
  return;
}


#define EPS 1E-8

int compare_content_Link(struct Link * const A, struct Link * const B)
{
  double x[3];

  x[0] = A->atom->ra - B->atom->ra;
  x[1] = A->atom->rb - B->atom->rb;
  x[2] = A->atom->rc - B->atom->rc;

  if(dot(x,x) < EPS) return 1;
  return 0;
}

struct Link * swap_Link(struct Link * A, struct Link * B)
{
  double key_tmp;
  struct Atom * atom_tmp;
  
  if(A == 0 || B == 0) {
    push_generic_error("copy_Link","(nil) args passed");
    exit_clean();
  }

  key_tmp = A->key;
  atom_tmp = A->atom;

  A->key = B->key;
  A->atom = B->atom;

  B->key = key_tmp;
  B->atom = atom_tmp;
  
  return A;

}
