/*

  (C) 1997 Marco Falcioni UCLA

  Here we parse the file and we create an array of scattering data
  based on the type of atoms.
  We use the label in the atoms to keep an index to the array.
  The array needs only n_unique elements.
  
  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "error.h"

#include "positions.h"
#include "link.h"
#include "chain.h"
#include "atoms.h"
#include "SP_entry.h"
#include "misc.h"
#include "verbose.h"
#include "molecule.h"

struct SP_entry ** read_SP_file(struct Positions * T, 
				char * name)
{
  int i, count = 0;
  struct SP_entry * me = 0;
  struct SP_entry ** SP;
  struct Link * aa, * bb;
  FILE * stream;
 
  if(T == 0 || name == 0) {
    push_generic_error("read_SP_file","(nil) args passed");
    exit_clean();
  }

  push_verbose_message("read_SP_file","reading scattering parameters table");
  
  stream = safe_open(name, "r");

  SP = (struct SP_entry **) Malloc((T->n_unique + T->Nmol_atoms) *
				   sizeof(struct SP_entry *));

  me = new_SP_entry();

  while(read_SP_entry(me, stream) != 0) {
    for(i = 0, aa = T->unique->head; aa != 0; aa = aa->next, i++) {
      int j, l; /* counters for molecules */
#ifdef DEBUG
      printf("Searching for atom %s, looking at %s\n",  
	     aa->atom->label, me->label);
#endif
      if(strcmp(me->label, aa->atom->label) == 0
	 && aa->atom->SP_tag == 0) {
	count ++;
	SP[i] = me;
	/* we found the line we want */
	/* we tag all the symmetry related atoms accordingly */
	for(bb = T->sym_related[i]->head; bb != 0; bb = bb->next) 
	  bb->atom->SP_tag = i;       
      }

      for(j = 0, l=0; j < T->Nmol; j++)  {/* molecule code */
	int k;
	for(k = 0; k < T->M[j]->N; l++, k++) {
	  if(strcmp(me->label, T->M[j]->A[k]->label) == 0
	     && T->M[j]->A[k]->SP_tag == 0) {
	    SP[l + T->n_unique] = me;
	    T->M[j]->A[k]->SP_tag = k + T->n_unique;
	  }
	}
      }

    }


    me = new_SP_entry();
  }

  if(count != T->unique->length) {
    push_generic_error("read_SP_file","some atoms are incorrectly labeled");
    exit_clean();
  }

  delete_SP_entry(me);

  fclose(stream);
  
  return SP;
}

