/*

  Marco Falcioni (C) 1997

  takes unique positions and generate symmetric copies
  then puts all copies back into the unit cell calling mod_Positions

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <string.h>

#include "error.h"
#include "positions.h"
#include "symmetry.h"
#include "atoms.h"
#include "chain.h"
#include "group.h"
#include "link.h"

/* generate all possible positions defined by the 
   action of the symmetry group */

void symmetrize_Positions(struct Positions * T, const struct Sym_group * G)
{
  int i = 0;
  struct Link * L;

  if(T==0 || G==0) {
    push_generic_error("symmetrize_Positions","(nil) args passed");
    exit_clean();
  }
  if(T->n_symmetry != G->n_el) {
    push_generic_error("symmetrize_Positions",
		       "mismatch found between positions and symmetry group");
    exit_clean();
  }


  /* 
     we assume that the identity is in position 0, which means
     that we ran test_Sym_group already...
  */

  if(G->test != 1) { /* warn that the group was not tested and the identity
			may not be element 0 */
    push_generic_error("symmetrize_Positions",
	   "Group not tested... the identity may not be first element");
  }

  /* generate all symmetric positions */
  for(i = 0 ; i < T->n_unique; i++) {
    struct Link * N;
    int j = 1;
    L = T->sym_related[i]->head;
    for(N = L->next; N != 0; N = N->next) {
      N->atom->root = L->atom->root;
      apply_Group_el(G->list[j], L->atom, N->atom);
      j++;
    }
  }

  T->test = 1; /* now this is a consistent set of positions */

  return;
}
