/*

  test the symmetry routines

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html


*/

#include <stdio.h>
#include <math.h>
#include "resources.h"
#include "stringlib.h"
#include "cell.h"
#include "group.h"
#include "positions.h"
#include "symmetry.h"
#include "params.h"
#include "rad_pattern.h"
#include "functions.h"
#include <error.h> /* from libgjm-1.3 */
#include "verbose.h"
#include "macros.h"
#include "grid.h"

int main (int argc, char ** argv)
{

  struct Params * P;
  struct Sym_group * G;
  struct Cell * C;
  struct Grid * Gr;
  struct Positions * T, *T1;
  struct string_list * options;
  struct Rad_pattern * RP = 0;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);

  read_data(&P, &T, &C, &G, &RP, stdin);

  Gr = new_Grid(P, C, 0);

  T1 = new_Positions(T->n_unique, T->n_symmetry, T->n_actual);

  oogl_Init_list(stdout);                     
  clone_Positions(T,T1);                    
  symmetrize_Positions(T1, G);              
  mod_Positions(T1);                        
  merge_positions(T1, P, C);                
  clean_Grid(Gr);                           
  fill_Grid(Gr, T1, C);                     
  oogl_Cell(C,0);                           
  oogl_bonds(P, C, T1, Gr,0);               
  oogl_Positions(T1, C);                    
  oogl_Finish_list();
  

  /*  write_Params(P, stdout);
  write_Cell(C,stdout);
  write_Sym_group(G, stdout);
  write_unique_Positions(T,stdout);
  */
  return 0;

}
