/*

  This program converts a "zefsa input file" to an XTL file

  
  (C) 1998 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <stdio.h>
#include <math.h>

#include "resources.h"
#include "stringlib.h"
#include "error.h" /* from libgjm-1.3 */

#include "cell.h"
#include "group.h"
#include "positions.h"
#include "symmetry.h"
#include "params.h"
#include "rad_pattern.h"
#include "functions.h"

#include "verbose.h"

int main (int argc, char ** argv)
{

  struct Params * P = 0;
  struct Sym_group * G = 0;
  struct Cell * C = 0;
  struct Positions * T = 0;
  struct string_list * options;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);

  P = read_Params(stdin);

  T = init_Positions(stdin);

  C = read_Cell(stdin);
 
  G = read_Sym_group(T->n_symmetry, stdin);

  test_Sym_group(G);

  if(C->csym == is_csym_Sym_group (G)) {
    /* no problem */
  } else {
    push_generic_error("read_data", "Centrosymmetric group/cell mismatch: check input");
    exit_clean();
  }

  read_Positions(T, stdin);
  
  find_polar_Sym_group(G);

  if(is_csym_Sym_group(G)) C->csym = 1;

  write_XTL(T,C,G,stdout);

  delete_Params(P);
  delete_Positions(T);
  delete_Cell(C);

  return 0;

}
